; *********************************************************************************
Temperatur_umrechnen:                   
;----------------------------------------------------------------------------------
    lds     Temp0, ID_Family_Code       ; DS18S20 oder DS18B20?
    
    cpi     Temp0, 0x28
    breq    DS18B20_Temperatur
    
    cpi     Temp0, 0x10
    breq    DS18S20_Temperatur

    ldi     Temp3, $FF                  ; unbekannter IC
    clr     MSB
    clr     LSB
    ret

; *********************************************************************************
DS18S20_Temperatur:

; Dieser IC hat nur 9 Bit mit 0,5C Auflsung ohne Zusatzaufwand 
;----------------------------------------------------------------------------------
;      MSB                                 LSB
;     --------------------------------- ; ---------------------------------
;     | S | S | S | S | S | S | S | S | ; | 6 | 5 | 4 | 3 | 2 | 1 | 0 | a |
;     --------------------------------- ; ---------------------------------
;     |<------- Vorzeichenbits ------>| ; |<--- Vorkommabits 6...0--->|Nachkomma|
; ---------------------------------------------------------------------------------
    lds     MSB, SP_Temp_MSB
    lds     LSB, SP_Temp_LSB
;----------------------------------------------------------------------------------
; Die DS18S20 Bits in DS18B20 Position bringen, drei Bits nach links schieben ber 2 Byte
    lsl     LSB
    rol     MSB
    lsl     LSB
    rol     MSB
    lsl     LSB
    rol     MSB
    andi    LSB, 0b11110000         ; alle Nachkommastellen lschen

; Auflsung erhhen durch Auswertung der Zusatzinfos
; Damit erreicht man die gleiche Auflsung wie DS18B20
;----------------------------------------------------------------------------------
;      MSB                              ;  LSB
;     --------------------------------- ; ---------------------------------
;     | S | S | S | S | S | 6 | 5 | 4 | ; | 3 | 2 | 1 | 0 | . | . | . | . |
;     --------------------------------- ; ---------------------------------
;     |<------------------| Vorkommabits 6....0 --------->| 0   0   0   0 |
;----------------------------------------------------------------------------------
;   TEMPERATURE = TEMP_READ - 0,25 + (COUNT_PER_C - COUNT_REMAIN) / COUNT_PER_C

    ldi     Temp1, 0x10                     ; COUNT_PER_C = 0x10
    lds     Temp2, SP_COUNT_REMAIN          ; COUNT_REMAIN = 0x10 - 0x01
    sub     Temp1, Temp2                    ; COUNT_PER_C - COUNT_REMAIN = $00 ... $0F
    or      LSB, Temp1                      ; LSB + (COUNT_PER_C- COUNT_REMAIN)
    subi    LSB, 4                          ; -0,25C / (1/16C) = -4
    sbc     MSB, NULL
    ret
    
; *********************************************************************************
DS18B20_Temperatur:
; Dieser IC hat schon immer die volle Auflsung von 12 Bit, wenn man sie passend einstellt
; nix gro zu tun, nur einlesen
;----------------------------------------------------------------------------------
;      MSB                              ;  LSB
;     --------------------------------- ; ---------------------------------
;     | S | S | S | S | S | 6 | 5 | 4 | ; | 3 | 2 | 1 | 0 | a | b | c | d |
;     --------------------------------- ; ---------------------------------
;     | Vorzeichen        | Vorkomma                  | Nachkomma     |
; ---------------------------------------------------------------------------------
    lds     MSB, SP_Temp_MSB
    lds     LSB, SP_Temp_LSB
    ret

; *********************************************************************************
Temperatur_anzeigen:                    ; Temperatur auf LCD ausgeben
; ---------------------------------------------------------------------------------
    ; Vorzeichen ermitteln und anzeigen, absoluten Betrag bilden

    ldi     Temp1, LCD_Line2        ; setze LCD auf Line2
    rcall   LCD_Cmd
    ldi     Temp1, '+'              ; positives Vorzeichen
    bst     MSB, 7                  
    brtc    Temperatur_anzeigen_plus; springe bei positiver Temperatur
    ldi     Temp1, '-'              ; negatives Vorzeichen
    com     LSB                     ; Temperatur invertieren
    com     MSB                     ; Bits invertieren
    ldi     temp0,1
    add     LSB, temp0
    adc     MSB, NULL               ; 1 addieren (16 Bit, Zweierkomplement)
Temperatur_anzeigen_plus:
    rcall   LCD_Data                ; schreibe Vorzeichen

    ; MSB und LSB passend zusammenschieben
    ; Vorzeichen + Vorkomma im MSB
    ; Nachkomma im LSB

    mov     temp0, LSB  
    swap    MSB                     ; SSSS S654 => S654 SSSS
    swap    LSB                     ; 3210 abcd => abcd 3210
    andi    MSB, 0xF0               ; S654 0000
    andi    LSB, 0x0F               ; 0000 3210
    or      MSB, LSB                ; S654 3210
    mov     LSB, temp0              ; 3210 abcd
    andi    LSB, 0x0F               ; 0000 abcd

; --------------------------------------
    ldi     Temp1, '0'
    cpi     MSB, 100                ; Temperatur >= 100C
    brlo    Temperatur_anzeigen_100
    ldi     Temp1, '1'              ; '1'
    subi    MSB, 100
Temperatur_anzeigen_100:
    rcall   LCD_Data                ; 100er Stelle ausgeben
    rcall   Ausgabe_Vorkomma        ; MSB dezimal 2-stellig auf LCD ausgeben
    ldi     Temp1, ','              ; ',' ausgeben
    rcall   LCD_Data
    rcall   Ausgabe_Nachkomma       ; LSB dezimal 1-stellig auf LCD ausgeben
    ldi     Temp1, 0xDF             ; '' ausgeben
    rcall   LCD_Data    
    ldi     Temp1, 'C'
    rcall   LCD_Data
    ret

; *********************************************************************************
Ausgabe_Vorkomma:                       ; Ausgabe von MSB als 2stellige Dezimalzahl
; ---------------------------------------------------------------------------------
    mov     temp2, MSB
    ldi     Temp1, '0'                  ; ASCII Code fr 1. Ziffer
Ausgabe_Vorkomma_schleife:              ; Anzahl zhlen, solange MSB >10 ist
    cpi     Temp2, 10                   ; Temp2 grsser als 10 ?
    brlo    Ausgabe_Vorkomma_Z1         ; springe, wenn niedriger
    inc     Temp1                       ; Ziffer erhhen
    subi    Temp2, 10                   ; 10 abziehen
    rjmp    Ausgabe_Vorkomma_schleife
Ausgabe_Vorkomma_Z1:    
    rcall   LCD_Data
                                        ; der Rest in temp2 ist die 2. Ziffer, 0-9
    subi    Temp2, -'0'                 ; + ASCII '0'
    mov     Temp1, Temp2
    rcall   LCD_Data    
    ret

; *********************************************************************************
Ausgabe_Nachkomma:
; Die Nachkommastelle LSB vom Binrbruch mit 1/16C Auflsung in einstellige Dezimalzahl umrechnen
; dezimal = LSB * 10 / 16
; Multiplikation ber Schieben und mehrfache Addition, Faktor 10 = 8 + 2 
; Division /16 durch 4 mal nach rechts schieben, geht schneller mit swap und Maskierung 
; ---------------------------------------------------------------------------------
    mov     temp1, LSB
    lsl     temp1           ; temp1 = LSB * 2
    mov     temp2, temp1
    lsl     temp2
    lsl     temp2           ; temp2 = LSB * 8
    add     temp1, temp2    ; temp1 = LSB * 10
    swap    temp1
    andi    temp1, 0x0F     ; temp1 = LSB * 10 / 16
    subi    temp1, -'0'     ; in ASCII umwandlen 
    rcall   LCD_Data    
    ret
