;**************************************************************************
;
;                   HD44780
;             LCD 1602 / 2004
;                    ----
;                   |    |
;            VCC -- |  1 | VSS
;            GND -- |  2 | VDD
;  GND -- 10kR <--- |  3 | VEE
;    PCF8574 P0 --> |  4 | RS
;    PCF8574 P1 --> |  5 | R/W
;    PCF8574 P2 --> |  6 | E
;                 x |  7 | D0
;                 x |  8 | D1
;                 x |  9 | D2
;                 x | 10 | D3
;    PCF8574 P4 --> | 11 | D4
;    PCF8574 P5 --> | 12 | D5
;    PCF8574 P6 --> | 13 | D6
;    PCF8574 P7 --> | 14 | D7
;   +5V -- 220R --> | 15 | A
;    PCF8574 P3 --> | 16 | C
;                   |    |
;                    ----
;
; -------------------------------------------------------------------------
; externes LCD ber I2C-Expander PCF8574 angesteuert
; Bitnummern der Signale                            
.equ    DP_RS               = 0
.equ    DP_RW               = 1
.equ    DP_E                = 2
.equ    DP_LED              = 3
.equ    DP_4                = 4
.equ    DP_5                = 5
.equ    DP_6                = 6
.equ    DP_7                = 7
; -------------------------------------------------------------------------
.equ    LCD_Reset           = 0b0011_1000   ; 0b0011_1... : LCD reset
.equ    LCD_4_Bit_Modus     = 0b0010_1000   ; 0b0010_1... : 4-Bit-Modus

.equ    LCD_Clear           = 0b0000_0001   ; Lscht Anzeige und Adresse = 0
.equ    LCD_Entry_Mode      = 0b0000_0110   ; Eingabemodus: Display rechtsdrehend
.equ    LCD_Display_On      = 0b0000_1100   ; Display On
.equ    LCD_FunctionSet     = 0b0010_1000   ; 4-bit data, 2-line display, 5x8 font
.equ    LCD_SetCursor       = 0b1000_0000   ; set cursor position

.equ    LCD_Line1           = 0x00 + LCD_SetCursor  ; start of line 1 (0x00..0x13)
.equ    LCD_Line2           = 0x40 + LCD_SetCursor  ; start of line 2 (0x40..0x53)
;**************************************************************************
LCD_Init:
; LCD initialisieren: Befehl muss 3 mal hintereinander gesendet werden
; -------------------------------------------------------------------------
    ldi     temp2, 15
    rcall   warte_Nx1ms             ; Wartezeit nach Power on

    ldi     Temp1, LCD_Reset        ; 1. LCD Reset : 0b0011_1000
    rcall   LCD_write_Nibble
    ldi     temp2, 5
    rcall   warte_Nx1ms

    rcall   LCD_write_Nibble        ; 2. LCD Reset : 0b0011_1000
    ldi     temp2, 1
    rcall   warte_Nx1ms

    rcall   LCD_write_Nibble        ; 3. LCD Reset : 0b0011_1000
    ldi     temp2, 1
    rcall   warte_Nx1ms

    ldi     Temp1, LCD_4_Bit_Modus  ; 4_Bit_Modus  : 0b0010_1000
    rcall   LCD_write_Nibble

    ldi     Temp1, LCD_FunctionSet  ; 4-bit data, 2-line display, 5x8 font
    rcall   LCD_Cmd     

    ldi     Temp1, LCD_Display_On   ; Display ein
    rcall   LCD_Cmd
 
    rcall   LCD_Clear_Display       ; Lscht Anzeige und Adresse = 0

    ldi     Temp1, LCD_Entry_Mode   ; Cursor Auto-Increment
    rcall   LCD_Cmd
    ret

;**************************************************************************
LCD_Clear_Display:                  ; Lscht Anzeige und Adresse = 0
; -------------------------------------------------------------------------
    ldi     Temp1, LCD_Clear
    rcall   LCD_Cmd
    ldi     temp2, 5
    rcall   warte_Nx1ms
    ret

;**************************************************************************
LCD_Cmd:                            ; ein Kommando an LCD senden
; Parameter: temp1: Kommando 
; -------------------------------------------------------------------------
    push    temp1                   ; temp1 zwischenspeichern
    andi    temp1, 0xF0             ; oberes Nibble maskieren
    rcall   LCD_write_Nibble        ; direkte Verbindung des oberen Nibbles zum LCD
    pop     temp1                   ; hole temp1 zurck
    swap    temp1                   ; Nibbles tauschen
    andi    temp1, 0xF0             ; oberes Nibble maskieren
    rcall   LCD_write_Nibble        ; direkte Verbindung des oberen Nibbles zum LCD
    ret

;**************************************************************************
LCD_Data:                           ; ein Byte an LCD senden
; Parameter: temp1: Daten
; -------------------------------------------------------------------------
    push    temp1                   ; temp1 zwischenspeichern
    andi    temp1, 0xF0             ; oberes Nibble maskieren
    sbr     temp1, (1<<DP_RS)       ; RS = High
    rcall   LCD_write_Nibble        ; direkte Verbindung des oberen Nibbles zum LCD
    pop     temp1                   ; hole temp1 zurck
    swap    temp1                   ; Nibbles tauschen
    andi    temp1, 0xF0             ; oberes Nibble maskieren
    sbr     temp1, (1<<DP_RS)       ; RS = High
    rcall   LCD_write_Nibble        ; direkte Verbindung des oberen Nibbles zum LCD
    ret

;**************************************************************************
LCD_write_Nibble:                   ; Steuerbyte an LCD senden, dabei E-Puls erzeugen
; Parameter: temp1: Daten
; -------------------------------------------------------------------------
    rcall   I2C_Start_Adresse
    sbrs    Licht, 0
    sbr     temp1, (1<<DP_LED)      ; LED EIN, low active
    rcall   I2C_Write
    sbr     temp1, (1<<DP_E)        ; LCD_E = High
    rcall   I2C_Write
    cbr     temp1, (1<<DP_E)        ; LCD_E = Low
    rcall   I2C_Write
    rcall   I2C_Stop
    ret
